/* $Id$ */
/* Copyright (c) 2011 Pierre Pronchery <khorben@defora.org> */
/* This file is part of DeforaOS System libc */
/* All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */



#include "../syscalls.h"
#include "sys/mount.h"


/* mount */
#if !defined(SYS_mount) && defined(SYS__mount) /* for Linux */
# include "stddef.h"
int _mount(char const * node, char const * dir, char const * type, int flags,
		char const * data);

int mount(char const * type, char const * dir, int flags, const void * data,
		size_t data_len)
{
	/* FIXME really implement */
	return _mount(NULL, dir, type, flags, NULL);
}
#elif !defined(SYS_mount)
# warning Unsupported platform: mount() is missing
# include "errno.h"
int mount(char const * type, char const * dir, int flags, const void * data,
		size_t data_len)
{
	errno = ENOSYS;
	return -1;
}
#endif


/* umount */
#ifndef SYS_unmount
# warning Unsupported platform: unmount() is missing
# include "errno.h"
int unmount(char const * dir, int flags)
{
	errno = ENOSYS;
	return -1;
}
#endif
